==> tailscale-1.20.0.tar.gz: tailscale-1.20.0/types/netmap/netmap_test.go <==
// Copyright (c) 2020 Tailscale Inc & AUTHORS All rights reserved.
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file.

package netmap

import (
	"encoding/hex"
	"testing"

	"go4.org/mem"
	"inet.af/netaddr"
	"tailscale.com/tailcfg"
	"tailscale.com/types/key"
)

func testNodeKey(b byte) (ret key.NodePublic) {
	var bs [key.NodePublicRawLen]byte
	for i := range bs {
		bs[i] = b
	}
	return key.NodePublicFromRaw32(mem.B(bs[:]))
}

func testDiscoKey(hexPrefix string) (ret key.DiscoPublic) {
	b, err := hex.DecodeString(hexPrefix)
	if err != nil {
		panic(err)
	}
	// this function is used with short hexes, so zero-extend the raw
	// value.
	var bs [32]byte
	copy(bs[:], b)
	return key.DiscoPublicFromRaw32(mem.B(bs[:]))
}

func TestNetworkMapConcise(t *testing.T) {
	for _, tt := range []struct {
		name string
		nm   *NetworkMap
		want string
	}{
		{
			name: "basic",
			nm: &NetworkMap{
				NodeKey: testNodeKey(1),
				Peers: []*tailcfg.Node{
					{
						Key:       testNodeKey(2),
						DERP:      "127.3.3.40:2",
						Endpoints: []string{"192.168.0.100:12", "192.168.0.100:12354"},
					},
					{
						Key:       testNodeKey(3),
						DERP:      "127.3.3.40:4",
						Endpoints: []string{"10.2.0.100:12", "10.1.0.100:12345"},
					},
				},
			},
			want: "netmap: self: [AQEBA] auth=machine-unknown u=? []\n [AgICA] D2                 :    192.168.0.100:12     192.168.0.100:12354\n [AwMDA] D4                 :       10.2.0.100:12        10.1.0.100:12345\n",
		},
		{
			name: "debug_non_nil",
			nm: &NetworkMap{
				NodeKey: testNodeKey(1),
				Debug:   &tailcfg.Debug{},
			},
			want: "netmap: self: [AQEBA] auth=machine-unknown u=? debug={} []\n",
		},
		{
			name: "debug_values",
			nm: &NetworkMap{
				NodeKey: testNodeKey(1),
				Debug:   &tailcfg.Debug{LogHeapPprof: true},
			},
			want: "netmap: self: [AQEBA] auth=machine-unknown u=? debug={\"LogHeapPprof\":true} []\n",
		},
	} {
		t.Run(tt.name, func(t *testing.T) {
			var got string
			n := int(testing.AllocsPerRun(1000, func() {
				got = tt.nm.Concise()
			}))
			t.Logf("Allocs = %d", n)
			if got != tt.want {
				t.Errorf("Wrong output\n Got: %q\nWant: %q\n## Got (unescaped):\n%s\n## Want (unescaped):\n%s\n", got, tt.want, got, tt.want)
			}
		})
	}
}

func TestConciseDiffFrom(t *testing.T) {
	for _, tt := range []struct {
		name string
		a, b *NetworkMap
		want string
	}{
		{
			name: "no_change",
			a: &NetworkMap{
				NodeKey: testNodeKey(1),
				Peers: []*tailcfg.Node{
					{
						Key:       testNodeKey(2),
						DERP:      "127.3.3.40:2",
						Endpoints: []string{"192.168.0.100:12", "192.168.0.100:12354"},
					},
				},
			},
			b: &NetworkMap{
				NodeKey: testNodeKey(1),
				Peers: []*tailcfg.Node{
					{
						Key:       testNodeKey(2),
						DERP:      "127.3.3.40:2",
						Endpoints: []string{"192.168.0.100:12", "192.168.0.100:12354"},
					},
				},
			},
			want: "",
		},
		{
			name: "header_change",
			a: &NetworkMap{
				NodeKey: testNodeKey(1),
				Peers: []*tailcfg.Node{
					{
						Key:       testNodeKey(2),
						DERP:      "127.3.3.40:2",
						Endpoints: []string{"192.168.0.100:12", "192.168.0.100:12354"},
					},
				},
			},
			b: &NetworkMap{
				NodeKey: testNodeKey(2),
				Peers: []*tailcfg.Node{
					{
						Key:       testNodeKey(2),
						DERP:      "127.3.3.40:2",
						Endpoints: []string{"192.168.0.100:12", "192.168.0.100:12354"},
					},
				},
			},
			want: "-netmap: self: [AQEBA] auth=machine-unknown u=? []\n+netmap: self: [AgICA] auth=machine-unknown u=? []\n",
		},
		{
			name: "peer_add",
			a: &NetworkMap{
				NodeKey: testNodeKey(1),
				Peers: []*tailcfg.Node{
					{
						ID:        2,
						Key:       testNodeKey(2),
						DERP:      "127.3.3.40:2",
						Endpoints: []string{"192.168.0.100:12", "192.168.0.100:12354"},
					},
				},
			},
			b: &NetworkMap{
				NodeKey: testNodeKey(1),
				Peers: []*tailcfg.Node{
					{
						ID:        1,
						Key:       testNodeKey(1),
						DERP:      "127.3.3.40:1",
						Endpoints: []string{"192.168.0.100:12", "192.168.0.100:12354"},
					},
					{
						ID:        2,
						Key:       testNodeKey(2),
						DERP:      "127.3.3.40:2",
						Endpoints: []string{"192.168.0.100:12", "192.168.0.100:12354"},
					},
					{
						ID:        3,
						Key:       testNodeKey(3),
						DERP:      "127.3.3.40:3",
						Endpoints: []string{"192.168.0.100:12", "192.168.0.100:12354"},
					},
				},
			},
			want: "+ [AQEBA] D1                 :    192.168.0.100:12     192.168.0.100:12354\n+ [AwMDA] D3                 :    192.168.0.100:12     192.168.0.100:12354\n",
		},
		{
			name: "peer_remove",
			a: &NetworkMap{
				NodeKey: testNodeKey(1),
				Peers: []*tailcfg.Node{
					{
						ID:        1,
						Key:       testNodeKey(1),
						DERP:      "127.3.3.40:1",
						Endpoints: []string{"192.168.0.100:12", "192.168.0.100:12354"},
					},
					{
						ID:        2,
						Key:       testNodeKey(2),
						DERP:      "127.3.3.40:2",
						Endpoints: []string{"192.168.0.100:12", "192.168.0.100:12354"},
					},
					{
						ID:        3,
						Key:       testNodeKey(3),
						DERP:      "127.3.3.40:3",
						Endpoints: []string{"192.168.0.100:12", "192.168.0.100:12354"},
					},
				},
			},
			b: &NetworkMap{
				NodeKey: testNodeKey(1),
				Peers: []*tailcfg.Node{
					{
						ID:        2,
						Key:       testNodeKey(2),
						DERP:      "127.3.3.40:2",
						Endpoints: []string{"192.168.0.100:12", "192.168.0.100:12354"},
					},
				},
			},
			want: "- [AQEBA] D1                 :    192.168.0.100:12     192.168.0.100:12354\n- [AwMDA] D3                 :    192.168.0.100:12     192.168.0.100:12354\n",
		},
		{
			name: "peer_port_change",
			a: &NetworkMap{
				NodeKey: testNodeKey(1),
				Peers: []*tailcfg.Node{
					{
						ID:        2,
						Key:       testNodeKey(2),
						DERP:      "127.3.3.40:2",
						Endpoints: []string{"192.168.0.100:12", "1.1.1.1:1"},
					},
				},
			},
			b: &NetworkMap{
				NodeKey: testNodeKey(1),
				Peers: []*tailcfg.Node{
					{
						ID:        2,
						Key:       testNodeKey(2),
						DERP:      "127.3.3.40:2",
						Endpoints: []string{"192.168.0.100:12", "1.1.1.1:2"},
					},
				},
			},
			want: "- [AgICA] D2                 :    192.168.0.100:12             1.1.1.1:1  \n+ [AgICA] D2                 :    192.168.0.100:12             1.1.1.1:2  \n",
		},
		{
			name: "disco_key_only_change",
			a: &NetworkMap{
				NodeKey: testNodeKey(1),
				Peers: []*tailcfg.Node{
					{
						ID:         2,
						Key:        testNodeKey(2),
						DERP:       "127.3.3.40:2",
						Endpoints:  []string{"192.168.0.100:41641", "1.1.1.1:41641"},
						DiscoKey:   testDiscoKey("f00f00f00f"),
						AllowedIPs: []netaddr.IPPrefix{netaddr.IPPrefixFrom(netaddr.IPv4(100, 102, 103, 104), 32)},
					},
				},
			},
			b: &NetworkMap{
				NodeKey: testNodeKey(1),
				Peers: []*tailcfg.Node{
					{
						ID:         2,
						Key:        testNodeKey(2),
						DERP:       "127.3.3.40:2",
						Endpoints:  []string{"192.168.0.100:41641", "1.1.1.1:41641"},
						DiscoKey:   testDiscoKey("ba4ba4ba4b"),
						AllowedIPs: []netaddr.IPPrefix{netaddr.IPPrefixFrom(netaddr.IPv4(100, 102, 103, 104), 32)},
					},
				},
			},
			want: "- [AgICA] d:f00f00f00f000000 D2 100.102.103.104 :   192.168.0.100:41641         1.1.1.1:41641\n+ [AgICA] d:ba4ba4ba4b000000 D2 100.102.103.104 :   192.168.0.100:41641         1.1.1.1:41641\n",
		},
	} {
		t.Run(tt.name, func(t *testing.T) {
			var got string
			n := int(testing.AllocsPerRun(50, func() {
				got = tt.b.ConciseDiffFrom(tt.a)
			}))
			t.Logf("Allocs = %d", n)
			if got != tt.want {
				t.Errorf("Wrong output\n Got: %q\nWant: %q\n## Got (unescaped):\n%s\n## Want (unescaped):\n%s\n", got, tt.want, got, tt.want)
			}
		})
	}
}

==> tailscale-1.32.0.tar.gz: tailscale-1.32.0/types/netmap/netmap_test.go <==
// Copyright (c) 2020 Tailscale Inc & AUTHORS All rights reserved.
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file.

package netmap

import (
	"encoding/hex"
	"net/netip"
	"testing"

	"go4.org/mem"
	"tailscale.com/net/netaddr"
	"tailscale.com/tailcfg"
	"tailscale.com/types/key"
)

func testNodeKey(b byte) (ret key.NodePublic) {
	var bs [key.NodePublicRawLen]byte
	for i := range bs {
		bs[i] = b
	}
	return key.NodePublicFromRaw32(mem.B(bs[:]))
}

func testDiscoKey(hexPrefix string) (ret key.DiscoPublic) {
	b, err := hex.DecodeString(hexPrefix)
	if err != nil {
		panic(err)
	}
	// this function is used with short hexes, so zero-extend the raw
	// value.
	var bs [32]byte
	copy(bs[:], b)
	return key.DiscoPublicFromRaw32(mem.B(bs[:]))
}

func TestNetworkMapConcise(t *testing.T) {
	for _, tt := range []struct {
		name string
		nm   *NetworkMap
		want string
	}{
		{
			name: "basic",
			nm: &NetworkMap{
				NodeKey: testNodeKey(1),
				Peers: []*tailcfg.Node{
					{
						Key:       testNodeKey(2),
						DERP:      "127.3.3.40:2",
						Endpoints: []string{"192.168.0.100:12", "192.168.0.100:12354"},
					},
					{
						Key:       testNodeKey(3),
						DERP:      "127.3.3.40:4",
						Endpoints: []string{"10.2.0.100:12", "10.1.0.100:12345"},
					},
				},
			},
			want: "netmap: self: [AQEBA] auth=machine-unknown u=? []\n [AgICA] D2                 :    192.168.0.100:12     192.168.0.100:12354\n [AwMDA] D4                 :       10.2.0.100:12        10.1.0.100:12345\n",
		},
		{
			name: "debug_non_nil",
			nm: &NetworkMap{
				NodeKey: testNodeKey(1),
				Debug:   &tailcfg.Debug{},
			},
			want: "netmap: self: [AQEBA] auth=machine-unknown u=? debug={} []\n",
		},
		{
			name: "debug_values",
			nm: &NetworkMap{
				NodeKey: testNodeKey(1),
				Debug:   &tailcfg.Debug{LogHeapPprof: true},
			},
			want: "netmap: self: [AQEBA] auth=machine-unknown u=? debug={\"LogHeapPprof\":true} []\n",
		},
	} {
		t.Run(tt.name, func(t *testing.T) {
			var got string
			n := int(testing.AllocsPerRun(1000, func() {
				got = tt.nm.Concise()
			}))
			t.Logf("Allocs = %d", n)
			if got != tt.want {
				t.Errorf("Wrong output\n Got: %q\nWant: %q\n## Got (unescaped):\n%s\n## Want (unescaped):\n%s\n", got, tt.want, got, tt.want)
			}
		})
	}
}

func TestConciseDiffFrom(t *testing.T) {
	for _, tt := range []struct {
		name string
		a, b *NetworkMap
		want string
	}{
		{
			name: "no_change",
			a: &NetworkMap{
				NodeKey: testNodeKey(1),
				Peers: []*tailcfg.Node{
					{
						Key:       testNodeKey(2),
						DERP:      "127.3.3.40:2",
						Endpoints: []string{"192.168.0.100:12", "192.168.0.100:12354"},
					},
				},
			},
			b: &NetworkMap{
				NodeKey: testNodeKey(1),
				Peers: []*tailcfg.Node{
					{
						Key:       testNodeKey(2),
						DERP:      "127.3.3.40:2",
						Endpoints: []string{"192.168.0.100:12", "192.168.0.100:12354"},
					},
				},
			},
			want: "",
		},
		{
			name: "header_change",
			a: &NetworkMap{
				NodeKey: testNodeKey(1),
				Peers: []*tailcfg.Node{
					{
						Key:       testNodeKey(2),
						DERP:      "127.3.3.40:2",
						Endpoints: []string{"192.168.0.100:12", "192.168.0.100:12354"},
					},
				},
			},
			b: &NetworkMap{
				NodeKey: testNodeKey(2),
				Peers: []*tailcfg.Node{
					{
						Key:       testNodeKey(2),
						DERP:      "127.3.3.40:2",
						Endpoints: []string{"192.168.0.100:12", "192.168.0.100:12354"},
					},
				},
			},
			want: "-netmap: self: [AQEBA] auth=machine-unknown u=? []\n+netmap: self: [AgICA] auth=machine-unknown u=? []\n",
		},
		{
			name: "peer_add",
			a: &NetworkMap{
				NodeKey: testNodeKey(1),
				Peers: []*tailcfg.Node{
					{
						ID:        2,
						Key:       testNodeKey(2),
						DERP:      "127.3.3.40:2",
						Endpoints: []string{"192.168.0.100:12", "192.168.0.100:12354"},
					},
				},
			},
			b: &NetworkMap{
				NodeKey: testNodeKey(1),
				Peers: []*tailcfg.Node{
					{
						ID:        1,
						Key:       testNodeKey(1),
						DERP:      "127.3.3.40:1",
						Endpoints: []string{"192.168.0.100:12", "192.168.0.100:12354"},
					},
					{
						ID:        2,
						Key:       testNodeKey(2),
						DERP:      "127.3.3.40:2",
						Endpoints: []string{"192.168.0.100:12", "192.168.0.100:12354"},
					},
					{
						ID:        3,
						Key:       testNodeKey(3),
						DERP:      "127.3.3.40:3",
						Endpoints: []string{"192.168.0.100:12", "192.168.0.100:12354"},
					},
				},
			},
			want: "+ [AQEBA] D1                 :    192.168.0.100:12     192.168.0.100:12354\n+ [AwMDA] D3                 :    192.168.0.100:12     192.168.0.100:12354\n",
		},
		{
			name: "peer_remove",
			a: &NetworkMap{
				NodeKey: testNodeKey(1),
				Peers: []*tailcfg.Node{
					{
						ID:        1,
						Key:       testNodeKey(1),
						DERP:      "127.3.3.40:1",
						Endpoints: []string{"192.168.0.100:12", "192.168.0.100:12354"},
					},
					{
						ID:        2,
						Key:       testNodeKey(2),
						DERP:      "127.3.3.40:2",
						Endpoints: []string{"192.168.0.100:12", "192.168.0.100:12354"},
					},
					{
						ID:        3,
						Key:       testNodeKey(3),
						DERP:      "127.3.3.40:3",
						Endpoints: []string{"192.168.0.100:12", "192.168.0.100:12354"},
					},
				},
			},
			b: &NetworkMap{
				NodeKey: testNodeKey(1),
				Peers: []*tailcfg.Node{
					{
						ID:        2,
						Key:       testNodeKey(2),
						DERP:      "127.3.3.40:2",
						Endpoints: []string{"192.168.0.100:12", "192.168.0.100:12354"},
					},
				},
			},
			want: "- [AQEBA] D1                 :    192.168.0.100:12     192.168.0.100:12354\n- [AwMDA] D3                 :    192.168.0.100:12     192.168.0.100:12354\n",
		},
		{
			name: "peer_port_change",
			a: &NetworkMap{
				NodeKey: testNodeKey(1),
				Peers: []*tailcfg.Node{
					{
						ID:        2,
						Key:       testNodeKey(2),
						DERP:      "127.3.3.40:2",
						Endpoints: []string{"192.168.0.100:12", "1.1.1.1:1"},
					},
				},
			},
			b: &NetworkMap{
				NodeKey: testNodeKey(1),
				Peers: []*tailcfg.Node{
					{
						ID:        2,
						Key:       testNodeKey(2),
						DERP:      "127.3.3.40:2",
						Endpoints: []string{"192.168.0.100:12", "1.1.1.1:2"},
					},
				},
			},
			want: "- [AgICA] D2                 :    192.168.0.100:12             1.1.1.1:1  \n+ [AgICA] D2                 :    192.168.0.100:12             1.1.1.1:2  \n",
		},
		{
			name: "disco_key_only_change",
			a: &NetworkMap{
				NodeKey: testNodeKey(1),
				Peers: []*tailcfg.Node{
					{
						ID:         2,
						Key:        testNodeKey(2),
						DERP:       "127.3.3.40:2",
						Endpoints:  []string{"192.168.0.100:41641", "1.1.1.1:41641"},
						DiscoKey:   testDiscoKey("f00f00f00f"),
						AllowedIPs: []netip.Prefix{netip.PrefixFrom(netaddr.IPv4(100, 102, 103, 104), 32)},
					},
				},
			},
			b: &NetworkMap{
				NodeKey: testNodeKey(1),
				Peers: []*tailcfg.Node{
					{
						ID:         2,
						Key:        testNodeKey(2),
						DERP:       "127.3.3.40:2",
						Endpoints:  []string{"192.168.0.100:41641", "1.1.1.1:41641"},
						DiscoKey:   testDiscoKey("ba4ba4ba4b"),
						AllowedIPs: []netip.Prefix{netip.PrefixFrom(netaddr.IPv4(100, 102, 103, 104), 32)},
					},
				},
			},
			want: "- [AgICA] d:f00f00f00f000000 D2 100.102.103.104 :   192.168.0.100:41641         1.1.1.1:41641\n+ [AgICA] d:ba4ba4ba4b000000 D2 100.102.103.104 :   192.168.0.100:41641         1.1.1.1:41641\n",
		},
	} {
		t.Run(tt.name, func(t *testing.T) {
			var got string
			n := int(testing.AllocsPerRun(50, func() {
				got = tt.b.ConciseDiffFrom(tt.a)
			}))
			t.Logf("Allocs = %d", n)
			if got != tt.want {
				t.Errorf("Wrong output\n Got: %q\nWant: %q\n## Got (unescaped):\n%s\n## Want (unescaped):\n%s\n", got, tt.want, got, tt.want)
			}
		})
	}
}
